#include <LiquidCrystal.h>
#include <Servo.h>

//SETUP
int wartezeit = 70;

//Servo Einstellungen (min - mitte - max ) 0 - 90 - 180 
int vert_mitte = 90;
int vert_min = 0;
int vert_max = 180;

int hori_mitte = 90;
int hori_min = 0;
int hori_max = 180;

//Schrittweite und Servorichtung 
int schrittweitehori = -1;
int schrittweitevert = -1;

//zum Kalibrieren der RSSI-Werte der Empfänger, dazu darf kein Sender angeschalten oder inder Nähe sein !
int cal_links = 0;
int cal_mitte = 0;
int cal_rechts = 0;
int cal_unten = 0;
int cal_oben = 0;

int hori=hori_mitte;
int vert=vert_max;
int last_hori=hori_mitte;
int last_vert=vert_max;

// Ausgabe Debug Infos auf Seriellen Port bzw. Serielles Display (SparkFun SerLCD V2.x)
boolean debug = true;

// Glättungszahl Abweichung links-recht bzw. oben-unten
int delta = 5;

// create servo object 
Servo servohori; 
Servo servovert; 

// Zuweisung analoge Eingänge
int rssi_links = A2;
int rssi_mitte = A4;
int rssi_rechts = A0;
int rssi_unten = A1;
int rssi_oben = A3;
#define SPKR 9



int links = 0;
int mitte = 0;
int rechts = 0;
int oben = 0;
int unten = 0;
int i = 0;
int hist[20] = {90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90,90};
int phist = 0;
int revert = 0;
int pause = 0;
int ser = 0;
int eff_schrittweitehori = 0;


void setup() {

      Serial.begin(9600);

  servohori.attach(9); // attaches the servo on pin 10 to the servo object
  servohori.write(hori_mitte);
  servovert.attach(10); // attaches the servo on pin 11 to the servo object
  servovert.write(vert_mitte);

  if (debug) {
    Serial.begin(9600);
    Serial.print("RSSI QuadTracker     V 0.01    ");
    delay(1000);
  }

  for(i=0;i<10;i++) {
    cal_links=cal_links+analogRead(rssi_links);
    delay(25);
  }
  cal_links=90-cal_links/10;
  // falls Calibrierung nicht sinnvoll (z.Bsp. Sender ist schon angeschalten) lieber 0
  if ( abs(cal_links) > 10 ) cal_links = 0;

  for(i=0;i<10;i++) {
    cal_mitte=cal_mitte+analogRead(rssi_mitte);
    delay(25);
  }
  cal_mitte=90-cal_mitte/10;
  // falls Calibrierung nicht sinnvoll (z.Bsp. Sender ist schon angeschalten) lieber 0
  if ( abs(cal_mitte) > 10 ) cal_mitte = 0;

  for(i=0;i<10;i++) {
    cal_rechts=cal_rechts+analogRead(rssi_rechts);
    delay(25);
  }
  cal_rechts=90-cal_rechts/10;
  // falls Calibrierung nicht sinnvoll (z.Bsp. Sender ist schon angeschalten) lieber 0
  if ( abs(cal_rechts) > 10 ) cal_rechts = 0;

  for(i=0;i<10;i++) {
    cal_unten=cal_unten+analogRead(rssi_unten);
    delay(25);
  }
  cal_unten=90-cal_unten/10;
  // falls Calibrierung nicht sinnvoll (z.Bsp. Sender ist schon angeschalten) lieber 0
  if ( abs(cal_unten) > 10 ) cal_unten = 0;

  for(i=0;i<10;i++) {
    cal_oben=cal_oben+analogRead(rssi_oben);
    delay(25);
  }
  cal_oben=90-cal_oben/10;
  // falls Calibrierung nicht sinnvoll (z.Bsp. Sender ist schon angeschalten) lieber 0
  if ( abs(cal_oben) > 10 ) cal_oben = 0;

  pinMode(SPKR, OUTPUT); //set the speaker as output
  digitalWrite(SPKR, HIGH);
  delay(500);
  digitalWrite(SPKR, LOW);
  delay(2500);

  if (debug) {
    Serial.print("                                ");
    Serial.write(254);
    Serial.write(129);
    Serial.print(cal_links);
    Serial.write(254);
    Serial.write(135);
    Serial.print(cal_oben);
    Serial.write(254);
    Serial.write(141);
    Serial.print(cal_rechts);
    Serial.write(254);
    Serial.write(192);
    Serial.print(cal_mitte);
    Serial.write(254);
    Serial.write(199);
    Serial.print(cal_unten);
  }
  delay(10000);

  if (debug) {
    Serial.print("                                ");
  }

}


void loop()
{

  // rssi_ einlesen
  links = analogRead(rssi_links) + cal_links;
  mitte = analogRead(rssi_mitte) + cal_mitte;
  rechts = analogRead(rssi_rechts) + cal_rechts;
  oben = analogRead(rssi_oben) + cal_oben;
  unten = analogRead(rssi_unten) + cal_unten;

  if (debug) {
    Serial.write(254);
    Serial.write(129);
    Serial.print(links);
    Serial.write(254);
    Serial.write(135);
    Serial.print(oben);
    Serial.write(254);
    Serial.write(141);
    Serial.print(rechts);
    Serial.write(254);
    Serial.write(192);
    Serial.print(mitte);
    Serial.write(254);
    Serial.write(199);
    Serial.print(unten);
  }

  //horizontal
  if (vert < vert_mitte) eff_schrittweitehori = schrittweitehori * -1;
  else  eff_schrittweitehori = schrittweitehori;
  if (mitte<links or mitte<rechts) {
    if (links > mitte)  hori = hori + eff_schrittweitehori;
    if (rechts > mitte) hori = hori - eff_schrittweitehori;
  }
  // weiterer Servoschritt wenn links und rechts nicht gleich
  if ( abs(links - rechts) > delta ) {
    if (links > rechts) hori = hori + eff_schrittweitehori;
    if (rechts > links) hori = hori - eff_schrittweitehori;
  }
  
  // vertikal
  if (mitte<oben or mitte<unten) {
    if (oben > mitte)  vert = vert + schrittweitevert;
    if (unten > mitte) vert = vert - schrittweitevert;
  }
  // weiterer Schritt wenn oben und unten nicht gleich
  if ( abs(oben - unten) > delta ) {
    if (oben > unten) vert = vert + schrittweitevert;
    if (unten > oben) vert = vert - schrittweitevert;
  }
  
  // debug - Display Infos
  if (debug) {
    // horizontal
    Serial.write(254);
    Serial.write(204);
    if ( last_hori > hori ) Serial.print("<");
    if ( last_hori < hori ) Serial.print(">");
    // vertikal
    Serial.write(254);
    Serial.write(207);
    if ( last_vert > vert ) Serial.print("^");
    if ( last_vert < vert ) Serial.print("v");
    last_vert = vert;
    last_hori = hori;
  }

  // Begrenzung Servoweg horizontal
  if (hori>hori_max) hori=hori_max;
  if (hori<hori_min) hori=hori_min;

  // Begrenzung Servowert vertikal
  if (vert>vert_max) vert=vert_max;
  if (vert<vert_min) vert=vert_min;

  // Servoumkehr ? Modell befindet sich ausserhalb des Drehbereiches des Horizontalservos
  // Historie einbeziehen letzte 10 Werte sind an der Grenze
  // nach Umkehr erstmal Pause 
  if ( pause < 0 ) {
    revert = hist[0];
    for(i = 1 ; i < 20 ; i++) revert = (revert + hist[i]) / 2;
    if (revert == hori_max) {
      hori = hori_min;
      if (vert > 90) vert = 90 - (vert - 90);
      else vert = 90 + (90 - vert);
      pause = 100;
    }
    if (revert == hori_min) {
      hori = hori_max;
      if (vert > 90) vert = 90 - (vert - 90);
      else vert = 90 + (90 - vert);
      pause = 100;
    }
  }
  else pause--;
  if (phist == 20) phist = 0;
  hist[phist] = hori;
  phist++;
  
  // Servo schreiben
  servohori.write(hori);
  servovert.write(vert);
  delay(wartezeit);

}
